import sys
from io import BytesIO
import logging

from .gpg import decrypt_resource_if_necessary

log = logging.getLogger('curlbomb.run')

def add_parser(subparsers):
    run_parser = subparsers.add_parser('run', help="Run a local script on the client")
    run_parser.add_argument('-c', '--command', metavar="COMMAND",
                            help="The the shell command to curlbomb into "
                            "(default is to detect #!interpreter ie. the shebang)",
                            default=None)
    run_parser.add_argument('resource', metavar="SCRIPT", nargs='?', default=sys.stdin)
    run_parser.set_defaults(prepare_command=prepare)


def prepare(args, settings, parser):
    settings['time_command'] = True
    settings['shell_command'] = args.command

    if args.resource == sys.stdin:
        args.resource = settings['stdin']
    
    if args.resource == settings['stdin'] and settings['stdin'].isatty():
        parser.print_help()
        sys.stderr.write("\nYou must specify a file or pipe one to this command's stdin\n")
        sys.exit(1)
    if settings.get('resource', None) is None:
        if args.resource == settings['stdin'] or args.resource == '-':
            settings['resource'] = BytesIO(settings['stdin'].buffer.read())
        else:
            settings['resource'] = open(args.resource, 'br')

    settings['resource'] = decrypt_resource_if_necessary(settings['resource'])
            
    #Detect if the input has a shebang so we can detect the shell command to display    
    if args.command is None:
        line = settings['resource'].readline(500)
        settings['resource'].seek(0)
        if line.startswith(b'#!'):
            settings['shell_command'] = line[2:].decode("utf-8").rstrip()
        else:
            settings['shell_command'] = "bash"
        



import logging
import subprocess
from io import BytesIO

log = logging.getLogger('curlbomb.gpg')

def detect_encrypted_resource(resource):
    """Detect if resource is encrypted with GnuPG

    Determining factors:
     a) The resource is ascii-armored text
     b) The resource is binary data starting with 0x8502"""
    line = resource.read(50)
    resource.seek(0)
    if line.startswith(b"-----BEGIN PGP"):
        return True
    elif line.startswith(b'\x85\x02'):
        return True
    else:
        return False
        
def decrypt_resource_if_necessary(resource):
    """Decrypt resource and return as decrypted BytesIO.

    If resource is not encrypted, return original resource"""
    if not detect_encrypted_resource(resource):
        return resource
    log.warn("Resource appears to be encrypted, handing off to gpg for decryption...")
    with subprocess.Popen(
            ['gpg','-d'], stdin=subprocess.PIPE,
            stdout=subprocess.PIPE, stderr=subprocess.PIPE) as p:
        p.stdin.write(resource.read())
        decrypted, stderr = p.communicate()
    if p.returncode != 0:
        raise RuntimeError("Resource failed decryption")
    decrypted_resource = BytesIO(decrypted)
    return decrypted_resource
